package bloques;

import java.util.*;

/**
 * <p>Title: BloqueBasico.java</p>
 * <p>Description: Clase para los bloques simples</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class BloqueBasico
    extends Bloque {
  static final long serialVersionUID = -7449682898656653653L;

  private double mttf = 0; // Tiempo medio hasta el fallo
  private double lambda = 0; // Tasa de fallos
  // Indica si el usuario ha introducido el mttf o el lambda
  private boolean asignadoMttf = false;
  private double mtbf = 0; // Tempo medio entre fallos

  private double mttr = 0; // Tiempo medio de reparacin
  private double mu = 0; // Tasa de reparacin
  // Indica si el usuario ha introducido el mttr o el mu
  private boolean asignadoMttr = true;

  /**Constructor de los bloques simples
   * @param iniName Es el nombre del nuevo bloque
   * @param padre_ Bloque padre del bloque a crear*/
  public BloqueBasico(String iniName, Bloque padre_) {
    super(iniName, padre_);
  }

  /**Calcula el mttf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mttf del bloque*/
  public double obtenerMttf(boolean reparable) {
    if (asignadoMttf) {
      return mttf;
    }
    else {
      return 1 / lambda;
    }
  }

  /**Calcula la tasa de fallos del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return La tasa de fallos del bloque*/
  public double obtenerLambda(boolean reparable) {
    if (asignadoMttf) {
      return 1 / mttf;
    }
    else {
      return lambda;
    }
  }

  /**Indica que parmetro ha sido definido para la fiabilidad del bloque
       * @return Cierto si se ha definido el mttf, falso si ha sido deinida la tasa
   * de fallos*/
  public boolean definidoMTTF() {
    return asignadoMttf;
  }

  /**Indica que parmetro ha sido definido para la reparabilidad del bloque
   * @return Cierto si se ha definido el mttr, falso si ha sido deinida la
   * tasa de reparacin*/
  public boolean definidoMTTR() {
    return asignadoMttr;
  }

  /**Asigna el valor del mttf
   * @param nuevoMttf Nuevo valor del mttf*/
  public void asignarMttf(double nuevoMttf) {
    mttf = nuevoMttf;
    asignadoMttf = true;
    mtbf = (obtenerMttr() + obtenerMttf(true));
    setChanged();
    notifyObservers();
  }

  /**Asigna el valor de la tasa de fallos
   * @param nuevoLambda Nuevo valor de la tasa de fallos*/
  public void asignarLambda(double nuevoLambda) {
    lambda = nuevoLambda;
    asignadoMttf = false;
    mtbf = (obtenerMttr() + obtenerMttf(true));
    setChanged();
    notifyObservers();
  }

  /**Asigna el valor del mtbf
   * @param nuevoMtbf Nuevo valor del mtbf*/
  public void asignarMtbf(double nuevoMtbf) {
    mtbf = nuevoMtbf;
  }

  /**Calcula el mttr del bloque
   * @return El Mttr del bloque*/
  public double obtenerMttr() {
    if (asignadoMttr) {
      return mttr;
    }
    else {
      return 1 / mu;
    }
  }

  /**Calcula el mtbf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mtbf del bloque*/
  public double obtenerMtbf(boolean reparable) {
    return mtbf;
  }

  /**Calcula la tasa de reparacin del bloque
   * @return La tasa de reparacin del bloque*/
  public double obtenerMu() {
    if (asignadoMttr) {
      return 1 / mttr;
    }
    else {
      return mu;
    }
  }

  /**Asigna el valor del mttr
   * @param nuevoMttr Nuevo valor del mttr*/
  public void asignarMttr(double nuevoMttr) {
    mttr = nuevoMttr;
    asignadoMttr = true;
    mtbf = (obtenerMttr() + obtenerMttf(true));
    setChanged();
    notifyObservers();
  }

  /**Asigna el valor de la tasa de reparacin
   * @param nuevoMu Nuevo valor de la tasa de reparacin*/
  public void asignarMu(double nuevoMu) {
    mu = nuevoMu;
    asignadoMttr = false;
    mtbf = (obtenerMttr() + obtenerMttf(true));
    setChanged();
    notifyObservers();
  }

  /**Calcula la fiabilidad del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @param t Valor del tiempo para el clculo de la fiabilidad
   * @return La fiabilidad del bloque*/
  public double obtenerFiabilidad(double t, boolean reparable) {
    return Math.exp( -obtenerLambda(reparable) * t);
  }

  /**Calcula la disponibilidad del bloque
   * @return La disponibilidad del bloque*/
  public double obtenerDisponibilidad() {
    if (obtenerLambda(true) <= 0) {
      throw new RuntimeException("Lambda es cero o menor que cero en bloque " +
                                 nombre());
    }
    if (obtenerMttr() <= 0) {
      throw new RuntimeException(" Mttr es cero o menor que cero en bloque " +
                                 nombre());
    }
    return (obtenerMu() / (obtenerLambda(true) + obtenerMu()));
  }

  /**Comprueba la validez para el clculo de un bloque
   * @param reparable Cierto si el bloque es reparable, falso en caso contrario
   * @return Un string que estar vaci si el diagrama est correcto para
   * realizar clculos y que en caso contrario contendr el motivo por el
   * cul no est correcto*/
  public String comprobar(boolean reparable) {
    String correcto = "";
    if (obtenerLambda(reparable) <= 0) {
      correcto = "El valor de lambda de bloque " + nombre() +
          " es menor o igual que cero";
    }
    else
    if (reparable && (obtenerMttr() <= 0)) {
      correcto = "El valor de mu de bloque " + nombre() +
          " es menor o igual que cero";
    }
    return correcto;
  }
}
